#ifndef EXAMPLE_HH
#define EXAMPLE_HH

#include "config.h"

#include <vector>

#include "graph_tool.hh"
#include "random.hh"
#include "inference/support/util.hh"
#include "inference/support/graph_state.hh"

namespace graph_tool
{
using namespace boost;
using namespace std;

// parameters to the description length
struct eargs_t
{
    bool param1 = false;
    double param2 = false;
};

typedef int64_t group_t;
typedef vprop_map_t<group_t> vmap_t;

#define EXAMPLE_STATE_params                                                   \
   ((g, &, decltype(all_graph_views), 1))                                      \
   ((b,, vmap_t, 0))

GEN_STATE_BASE(ExampleStateBase, EXAMPLE_STATE_params)

template <class... Ts>
class ExampleState
    : public ExampleStateBase<Ts...>
{
public:
    GET_PARAMS_USING(ExampleStateBase<Ts...>, EXAMPLE_STATE_params)
    GET_PARAMS_TYPEDEF(Ts, EXAMPLE_STATE_params)
    using typename ExampleStateBase<Ts...>::args_t;
    using ExampleStateBase<Ts...>::dispatch_args;

    template <class... ATs,
              typename std::enable_if_t<sizeof...(ATs) == sizeof...(Ts)>* = nullptr>
    ExampleState(ATs&&... args)
        : ExampleStateBase<Ts...>(std::forward<ATs>(args)...)
    {
        // the paramters here are accessible as members with underscores, e.g.
        // _g, _b, etc.
    }

    // Computes the DL difference of moving vertex from group r to nr
    double virtual_move(size_t v, size_t r, size_t nr, eargs_t& ea)
    {
        return 0;
    }

    // Moves vertex v to group s
    void move_vertex(size_t v, size_t s)
    {
        return;
    }

    // Sample a new group membership for vertex v
    group_t sample_group(size_t v, rng_t& rng)
    {
        return 0;
    }

    // Calculate the log-probability of moving vertex v from group r to s (or
    // the reverse probability if reverse == true)
    double get_move_lprob(size_t v, group_t r, group_t s,
                          bool reverse)
    {
        // warning: if reverse == true, the probability returned should match
        // the state which is identical to the current one *except* that the
        // membership of vertex v should be considered to be equal to s, and the
        // proposal would move it to r.
        return 0;
    }

    double entropy(const eargs_t& ea)
    {
        return 0;
    }
};
} // graph_tool namespace

#endif //EXAMPLE_HH
