import graph_tool
from graph_tool import Graph, GraphView, _get_rng, _prop, PropertyMap
from graph_tool.inference.base_states import EntropyState, MCMCState, \
    MultiflipMCMCState, MultilevelMCMCState, DrawBlockState, copy_state_wrap, \
    entropy_state_signature
from graph_tool.inference.util import *
import numpy as np

import libexample

@entropy_state_signature
class ExampleState(MCMCState, MultiflipMCMCState, MultilevelMCMCState,
                   DrawBlockState):
    r"""FIXME: write the documentation"""

    def __init__(self, g, b=None, entropy_args={}, **kwargs):
        EntropyState.__init__(self, entropy_args=entropy_args)
        self.g = g
        if b is None:
            b = g.new_vp("int64_t")
        self.b = b
        self._state = libexample.make_example_state(self)

    def __copy__(self):
        return self.copy()

    def copy(self, **kwargs):
        r"""Copies the state. The parameters override the state properties, and
         have the same meaning as in the constructor."""
        args = self.__getstate__()
        args.update(**kwargs)
        return ExampleState(**args)

    def __getstate__(self):
        state = EntropyState.__getstate__(self)
        return dict(state, g=self.g, b=self.b)

    def __setstate__(self, state):
        self.__init__(**state)

    @copy_state_wrap
    def _entropy(self, peram1=False, param2=42., **kwargs):
        r"""Return the description length (negative joint log-likelihood)."""

        eargs = self._get_entropy_args(locals())

        S = self._state.entropy(eargs, False)

        if len(kwargs) > 0:
            raise ValueError("unrecognized keyword arguments: " +
                             str(list(kwargs.keys())))
        return S

    def _gen_eargs(self, args):
        return libexample.eargs()

    def _mcmc_sweep_dispatch(self, mcmc_state):
        return libexample.example_mcmc_sweep(mcmc_state, self._state,
                                             _get_rng())

    def _multiflip_mcmc_sweep_dispatch(self, mcmc_state):
        return libexample.example_multiflip_mcmc_sweep(mcmc_state,
                                                       self._state,
                                                       _get_rng())

    def _multilevel_mcmc_sweep_dispatch(self, mcmc_state):
        return libexample.example_multilevel_mcmc_sweep(mcmc_state,
                                                        self._state,
                                                        _get_rng())
