#include "graph_tool.hh"
#include "random.hh"

#include <boost/python.hpp>

#include "example.hh"
#include "example_mcmc.hh"

using namespace boost;
using namespace graph_tool;

GEN_DISPATCH(example_state, ExampleState, EXAMPLE_STATE_params)

template <class State>
GEN_DISPATCH(mcmc_example_state, MCMC<State>::template MCMCExampleState,
             MCMC_EXAMPLE_STATE_params(State))

python::object example_mcmc_sweep(python::object omcmc_state,
                                  python::object oexample_state,
                                  rng_t& rng)
{
    python::object ret;
    example_state::dispatch
        (oexample_state,
         [&](auto& example_state)
         {
             typedef typename std::remove_reference<decltype(example_state)>::type
                 state_t;

             mcmc_example_state<state_t>::make_dispatch
                 (omcmc_state,
                  [&](auto& s)
                  {
                      auto ret_ = mcmc_sweep(*s, rng);
                      ret = tuple_apply([&](auto&... args){ return python::make_tuple(args...); }, ret_);
                  });
         });
    return ret;
}

#define __MOD__ example
#include "module_registry.hh"
REGISTER_MOD
([]
{
    using namespace boost::python;
    def("example_mcmc_sweep", &example_mcmc_sweep);
});
