// graph-tool -- a general graph modification and manipulation thingy
//
// Copyright (C) 2006-2025 Tiago de Paula Peixoto <tiago@skewed.de>
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3 of the License, or (at your option) any
// later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.

#include "graph_tool.hh"
#include "random.hh"

#include <boost/python.hpp>

#include "example.hh"
#include "example_multiflip_mcmc.hh"
#include "inference/loops/mcmc_loop.hh"

using namespace boost;
using namespace graph_tool;

GEN_DISPATCH(example_state, ExampleState, EXAMPLE_STATE_params)

template <class State>
GEN_DISPATCH(mcmc_example_state, MCMC<State>::template MCMCExampleState,
             MCMC_EXAMPLE_STATE_params(State))

python::object example_multiflip_mcmc_sweep(python::object omcmc_state,
                                            python::object oexample_state,
                                            rng_t& rng)
{
    python::object ret;
    example_state::dispatch
        (oexample_state,
         [&](auto& example_state)
         {
             typedef typename std::remove_reference<decltype(example_state)>::type
                 state_t;
             mcmc_example_state<state_t>::make_dispatch
                 (omcmc_state,
                  [&](auto& s)
                  {
                      auto ret_ = mcmc_sweep(*s, rng);
                      ret = tuple_apply([&](auto&... args){ return python::make_tuple(args...); }, ret_);
                  });
         });
    return ret;
}

#define __MOD__ example
#include "module_registry.hh"
REGISTER_MOD
([]
{
    using namespace boost::python;
    def("example_multiflip_mcmc_sweep", &example_multiflip_mcmc_sweep);
});
