#ifndef EXAMPLE_MULTILEVEL_MCMC_HH
#define EXAMPLE_MULTILEVEL_MCMC_HH

#include "config.h"

#include <vector>
#include <algorithm>

#include "graph_tool.hh"
#include "inference/support/graph_state.hh"

#include "example.hh"

#include "idx_map.hh"
#include "inference/loops/multilevel.hh"

namespace graph_tool
{
using namespace boost;
using namespace std;

typedef vprop_map_t<int64_t> vmap_t;

#define MCMC_EXAMPLE_STATE_params(State)                                       \
    ((__class__,&, decltype(hana::tuple_t<python::object>), 1))                \
    ((state, &, State&, 0))                                                    \
    ((beta,, double, 0))                                                       \
    ((r,, double, 0))                                                          \
    ((random_bisect,, bool, 0))                                                \
    ((merge_sweeps,, size_t, 0))                                               \
    ((mh_sweeps,, size_t, 0))                                                  \
    ((parallel,, bool, 0))                                                     \
    ((init_min_iter,, size_t, 0))                                              \
    ((init_r,, double, 0))                                                     \
    ((init_beta,, double, 0))                                                  \
    ((gibbs,, bool, 0))                                                        \
    ((M,, size_t, 0))                                                          \
    ((global_moves,, bool, 0))                                                 \
    ((cache_states,, bool, 0))                                                 \
    ((B_min,, size_t, 0))                                                      \
    ((B_max,, size_t, 0))                                                      \
    ((b_min,, vmap_t, 0))                                                      \
    ((b_max,, vmap_t, 0))                                                      \
    ((force_accept,, bool, 0))                                                 \
    ((entropy_args,, eargs_t, 0))                                              \
    ((verbose,, int, 0))                                                       \
    ((niter,, size_t, 0))

template <class State>
struct MCMC
{
    GEN_STATE_BASE(MCMCExampleStateBase, MCMC_EXAMPLE_STATE_params(State))

    template <class... Ts>
    class MCMCExampleStateImp
        : public MCMCExampleStateBase<Ts...>,
          public MultivelStateBase
    {
    public:
        GET_PARAMS_USING(MCMCExampleStateBase<Ts...>,
                         MCMC_EXAMPLE_STATE_params(State))
        GET_PARAMS_TYPEDEF(Ts, MCMC_EXAMPLE_STATE_params(State))

        template <class... ATs,
                  typename std::enable_if_t<sizeof...(ATs) ==
                                            sizeof...(Ts)>* = nullptr>
        MCMCExampleStateImp(ATs&&... as)
        : MCMCExampleStateBase<Ts...>(as...)
        {
            if (_global_moves)
            {
                idx_set<group_t> rs_min, rs_max;
                for (auto v : vertices_range(_state._g))
                {
                    rs_min.insert(_b_min[v]);
                    rs_max.insert(_b_max[v]);
                }
                _has_b_min = rs_min.size() == _B_min;
                _has_b_max = rs_max.size() == _B_max;
            }
        }

        bool _has_b_max = false;
        bool _has_b_min = false;

        idx_set<group_t> _rs;

        constexpr static group_t _null_group = std::numeric_limits<group_t>::max();

        template <class F>
        void iter_nodes(F&& f)
        {
            for (auto v : vertices_range(_state._g))
                f(v);
        }

        template <class F>
        void iter_groups(F&& f)
        {
            // FIXME: This should iterate overa all groups and call the function
            // f on each of them
        }

        auto get_group(size_t v)
        {
            return _state._b[v];
        }

        template <class RNG>
        group_t get_new_group(size_t v, bool, RNG& rng)
        {
            // FIXME: This should return a new, empty group.
            return 0;
        }

        void move_node(size_t v, group_t r, bool)
        {
            _state.move_vertex(v, r);
        }

        double virtual_move(size_t v, group_t r, group_t s)
        {
            return _state.virtual_move(v, r, s, _entropy_args);
        }

        group_t get_b_min(size_t v)
        {
            return _b_min[v];
        }

        group_t get_b_max(size_t v)
        {
            return _b_max[v];
        }

        template <class RNG>
        group_t sample_group(size_t v, bool allow_random, bool,
                             bool init_heuristic, RNG& rng)
        {
            return _state.sample_group(v, rng);
        }

        double get_move_prob(size_t v, group_t r, group_t s, bool allow_random,
                             bool, bool reverse)
        {
            return _state.get_move_lprob(v, r, s, reverse);
        }

        double entropy()
        {
            return _state.entropy(_entropy_args);
        }
    };

    using gmap_t = idx_map<group_t, idx_set<size_t, true>, false, true, true>;

    template <class T>
    using iset = idx_set<T>;

    template <class T, class V>
    using imap = idx_map<T, V>;

    template <class... Ts>
    class MCMCExampleState:
        public Multilevel<MCMCExampleStateImp<Ts...>,
                          size_t,
                          group_t,
                          iset,
                          imap,
                          iset,
                          imap,
                          gmap_t, false>
    {
    public:
        template <class... ATs,
                  typename std::enable_if_t<sizeof...(ATs) ==
                                            sizeof...(Ts)>* = nullptr>
        MCMCExampleState(ATs&&... as)
           : Multilevel<MCMCExampleStateImp<Ts...>,
                        size_t,
                        group_t,
                        iset,
                        imap,
                        iset,
                        imap,
                        gmap_t, false>(as...)
        {}
    };
};

} // graph_tool namespace

#endif //EXAMPLE_MULTILEVEL_MCMC_HH
