---
kernelspec:
  name: python3
  display_name: 'Python 3'
---

# Exercise 2: Implementing a simple SBM state and MCMC

**Authors:** Tiago P. Peixoto<sup>1</sup> \
**Affiliations:** <sup>1</sup>Inverse Complexity Lab \
**License:** CC-BY

The idea of this exercise is to implement the inference of a simple planted
partition SBM with a likelihood given by
```{math}
:label: sbm
P(A|\lambda_{\text{in}},\lambda_{\text{out}}, \bm b)
= \prod_{ij} \frac{\mathrm{e}^{-\lambda_{ij}}\lambda_{ij}^{A_{ij}}}{A_{ij}!},
```
with
$$
\lambda_{ij} = \lambda_{\text{in}}\delta_{b_i,b_j} + \lambda_{\text{out}}(1-\delta_{b_i,b_j}).
$$

The templates below (available from the manual
[repo](https://git.skewed.de/invcomplexity/lab-manual); no need to copy and
paste.) can be used to make use of the functionalities already available in
graph-tool.

The first file defines a C++ state object that receives a series of parameters.

```{include} example.hh
:filename: example.hh
:lang: cpp
```

:::{important}
The file above needs to be modified to correspond to our inference problem above!

The comments explain what each function needs to do.
:::

The second file is a compilation unit that takes care of instantiating the state
above when this is requested from Python. This is just boilerplate, and does not
need to be modified.

```{include} example.cc
:filename:  example.cc
:lang: cpp
```

The MCMC itself requires us to implement the following template, which is
relatively straightforward:

```{include} example_mcmc.hh
:filename: example_mcmc.hh
:lang: cpp
```

As well as its compilation unit:

```{include} example_mcmc.cc
:filename: example_mcmc.c
:lang: cpp
```

Finally, we need a a Python file that binds everything from the C++ side.

```{include} example.py
:filename:  example.py
:lang: python
```

The ``Makefile`` below facilitates the compilation:

```{include} Makefile
:filename:  Makefile
:lang: make
```

We need to compile the C++ code by typing ``make``:

```bash
$ make
```
```{code-cell} python
:tags: [remove-input]
!make -j 4
```

Now we are ready to call the function above from Python:

```{code-cell} python
from graph_tool.all import *
from example import ExampleState

g = collection.data["dolphins"]

state = ExampleState(g)
state.mcmc_sweep()
```

# Task 1: Modify the code above to implement the SBM of Eq [](#sbm).

Add a link here to your solution as a gitlab repository.

# Task 2: Modify the code below to implement a merge-split MCMC

```{include} example_multiflip_mcmc.hh
:filename: example_multiflip_mcmc.hh
:lang: cpp
```

Add a link here to your solution as a gitlab repository.

# Task 3: Modify the code below to implement an agglomerative heuristic

```{include} example_multilevel_mcmc.hh
:filename: example_multilevel_mcmc.hh
:lang: cpp
```

Add a link here to your solution as a gitlab repository.
