---
kernelspec:
  name: python3
  display_name: 'Python 3'
---

# The Inverse Complexity Lab Manual

**Authors:** Tiago P. Peixoto<sup>1</sup> \
**Affiliations:** <sup>1</sup>Inverse Complexity Lab \
**License:** CC-BY

This site documents essential technical knowledge with which lab members are
encouraged and expected to familiarize themselves. It is also meant as a
platform for lab members to share knowledge, and form a common technical
language.

::::{aside}
:::{tip} Get automatically notified of changes made here!

You can get notified about changes made to this manual by following the git
repository at https://git.skewed.de/invcomplexity/lab-manual, and changing your
notification settings to "watch". This is maybe the best way to be up to date.

The same applies to other repositories such as
https://git.skewed.de/count0/graph-tool, or any other.
:::
::::
:::{important}
# This is a living document!

Lab members are expected to contribute to it by making improvements and expanding
its contents.
:::


This site is built with [myst](https://mystmd.org) and is available from its [gitlab
repository](https://git.skewed.de/invcomplexity/lab-manual). To modify it locally just clone the repository with

```bash
git clone https://git.skewed.de/invcomplexity/lab-manual
```

After installing [myst](https://mystmd.org) locally, the website can be
previewed with:

```bash
cd lab-manual
myst start --execute
```

The content is written in Markdown, which has a very easy syntax. Changes made
to the content files are reflected automatically, which makes editing a rather
pleasant experience. For changes to be reflected online, fork the repository in
gitlab, push your changes, and open a merge request. Once the request is
accepted, the changes will go live.

Latex notation is supported for mathematics, e.g.
$$
{N \choose m } = \frac{N!}{m!(N-m)!}.
$$
In fact myst supports very rich authoring features, including cross-references,
bibliography, figures, equations, etc. Take a look at the
[documentation](https://mystmd.org/guide) for more details.

In particular, it's possible to run Python code very much like a jupyter notebook:
```{code-cell} python
import graph_tool.all as gt
import matplotlib.cm

gt.seed_rng(47)

g = gt.collection.ns["foodweb_baywet"]

state = gt.minimize_nested_blockmodel_dl(g, state_args=dict(recs=[g.ep.weight],
                                                            rec_types=["real-exponential"]))

state.draw(edge_color=g.ep.weight,
           ecmap=(matplotlib.cm.inferno, .6),
           ecnorm=matplotlib.colors.LogNorm(g.ep.weight.fa.min(), g.ep.weight.fa.max()),
           eorder=g.ep.weight,
           edge_pen_width=gt.prop_to_size(g.ep.weight, 1, 4, power=1, log=True),
           edge_gradient=[]);
```

Code results are automatically cached across runs, so it does not break with the
interactivity. Indeed, this works as an interesting alternative to notebooks,
since you can use your text editor, instead of the browser, and do not have to
worry about cells, etc.
